//
// Source: kmaze1.bas
// Translated from Basic to C++ using btran(~tm:)
// on Monday, July 10, 2000 at 12:11:02
//

#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include "basicfun.h"
#include "bstring.h"
#include "basicchannel.h"



//
// Function Prototypes
//


int basic_main()
{
	Int32 avail;
	Int32 availpath[11];
	Int32 path;
	Int32 unused;
	Int32 x;
	Int32 xmax;
	Int32 y;
	Int32 ymax;

	Int32 walls[101][101];
	//
	// KMAZE.BAS
	//
	//	A simple maze generating program
	//
	// Description:
	//
	//	This is a simple maze generating program to show how
	//	to generate two dimensional mazes in BASIC.
	//
	//	This version uses the last added cell as the base
	//	to append to, and thus generates a better looking
	//	maze.
	//
	// Algorithm:
	//
	//	1. Mark all walls around the cells as being 'up'.
	//	2. Pick one cell at random above the top row,
	//	   and the cell below it as the destination.
	//	3. Mark the wall between the two selected cells
	//	   as 'down'.
	//	4. If all cells are used, then were finished.
	//	5. Make the destination cell as the current cell.
	//	6. Randomally select any cell in the four adjacent
	//	   sides as yhe cell to move to.
	//	   If there is such a cell, go to step 3.
	//	7. Pick a cell at random, which has been connected
	//	   to at least one cell, and has an open cell in one
	//	   of the four adjacent cells.
	//	8. Goto 3
	//
	// Enviornment:
	//
	//	VAX/BASIC on VAX/VMS
	//
	// Copyright:
	//
	//	I place this code in the public domain, with the
	//	provision that my name remain attached to the code when it
	//	is used in any additional program.
	//
	//	This code is given for demonstration purposes only, and
	//	is simplified from what would be required for a commercial
	//	program.
	//
	//	No warranties, express or implied, is granted. This code
	//	is supplied for educational purposes only.  It is the
	//	users responsibility to make sure that this code works
	//	appropriately for any given task.
	//
	// History:
	//
	//	08/15/1997 - Kevin Handy <kth@srv.net>
	//		Software Solutions, Inc., Idaho Falls, Idaho, USA
	//		Original version
	//
	//
	// WALLS%() Contains a reference to which walls have been taken
	// down from the cell. If the cell has a zero value, then the
	// cell isn't included in a path yet (no broken walls).
	//	1 - up		2 - down
	//	4 - left	8 - right
	//
	// Array size limits maze size
	// Should really be defined
	srand(time(0));
	BasicChannel[0] << "MAZE SIZE (X,Y)";
	BasicChannel[0] >> xmax >> ymax;
	//
	// Initialize the maze to having all walls up
	//
	for (y = 1; y <= ymax; y++)
	{
		for (x = 1; x <= xmax; x++)
		{
			walls[x][y] = 0;
		}
	}
	unused = xmax * ymax;
	//
	// Randomally pick a starting cell on the first row,
	// to get things started and tell it we're going up
	//
	x = 0;
	y = floor(floatrand() * ymax) + 1;
	avail = 1;
	availpath[avail] = 2;
	path = 1;
	//
L_140:;
	// Mark the path taken
	//
	switch (availpath[path])
	{
		//
		// Going up
		//
	case 1:

		walls[x][y] = walls[x][y] | 1;
		walls[x - 1][y] = walls[x - 1][y] | 2;
		x = x - 1;
		//
		// Going down
		//
		break;

	case 2:

		walls[x][y] = walls[x][y] | 2;
		walls[x + 1][y] = walls[x + 1][y] | 1;
		x = x + 1;
		//
		// Going left
		//
		break;

	case 4:

		walls[x][y] = walls[x][y] | 4;
		walls[x][y - 1] = walls[x][y - 1] | 8;
		y = y - 1;
		//
		// Going right
		//
		break;

	case 8:

		walls[x][y] = walls[x][y] | 8;
		walls[x][y + 1] = walls[x][y + 1] | 4;
		y = y + 1;
		break;

	}
	unused = unused - 1;
	//
	// Now, we need to look for another available path
	//
	if (unused == 0)
	{
		goto L_200;
	}
	//
L_160:;
	// Having an (X,Y) location with at least one open path,
	// Create a list of available directions
	//
	avail = 0;
	if ((x > 1) && walls[x - 1][y] == 0)
	{
		avail = avail + 1;
		availpath[avail] = 1;
	}
	if ((x < xmax) && walls[x + 1][y] == 0)
	{
		avail = avail + 1;
		availpath[avail] = 2;
	}
	if ((y > 1) && walls[x][y - 1] == 0)
	{
		avail = avail + 1;
		availpath[avail] = 4;
	}
	if ((y < ymax) && walls[x][y + 1] == 0)
	{
		avail = avail + 1;
		availpath[avail] = 8;
	}
	//
	// If we haven't got somewhere to go, look at the next square
	//
	if (avail == 0)
	{
		goto L_190;
	}
	//
	// Now, choose one of the available directions for the
	// next site to connect to
	//
	path = floatrand() * avail + 1;
	goto L_140;
	//
L_190:;
	// Randomally pick a starting cell to get things started
	//
	x = floor(floatrand() * xmax) + 1;
	y = floor(floatrand() * ymax) + 1;
	//
	// Look for a connected square adjacent to an open square
	//
	while (walls[x][y] == 0 || (walls[x + 1][y] != 0 && walls[x - 1][y] != 0 && walls[x][y + 1] != 0 && walls[x][y - 1] != 0))
	{
		x = x + 1;
		if (x > xmax)
		{
			x = 1;
			y = y + 1;
			if (y > ymax)
			{
				y = 1;
			}
		}
	}
	goto L_160;
	//
L_200:;
	// Now, print out the maze
	//
	for (x = 1; x <= xmax; x++)
	{
		//
		// Box Tops
		//
		for (y = 1; y <= ymax; y++)
		{
			if ((walls[x][y] & 1) == 0)
			{
				BasicChannel[0] << ":--";
			}
			else
			{
				BasicChannel[0] << ":  ";
			}
		}
		BasicChannel[0] << ":" << endl;
		//
		// Left sides
		//
		for (y = 1; y <= ymax; y++)
		{
			if ((walls[x][y] & 4) == 0)
			{
				BasicChannel[0] << "I  ";
			}
			else
			{
				BasicChannel[0] << "   ";
			}
		}
		BasicChannel[0] << "I" << endl;
	}
	//
	// Box Tops
	//
	x = floatrand() * ymax + 1;
	for (y = 1; y <= ymax; y++)
	{
		if (y != x)
		{
			BasicChannel[0] << ":--";
		}
		else
		{
			BasicChannel[0] << ":  ";
		}
	}
	BasicChannel[0] << ":" << endl;

	return EXIT_SUCCESS;
}
