//
// basicchannel.h
//
//	Header file for Basic+ I/O
//
// Description:
//
//	Class to handle Basic channels.
//
//	This class is used in a program (usually as an array of
//	channels) to handle the BASIC I/O statements.
//
//	It tries to map as much of the work through C++ streams
//	as possible, so that converting to pure C++ will be
//	easier.
//
// History
//
//	03/21/1997 - Kevin Handy
//		Original version.
//
//	04/09/1997 - Kevin Handy
//		Pulled in stuff from basicfun.h
//
//	12/06/1998 - Kevin Handy
//		Start some real work on this.
//		Make it handle C++ streams by defining istream/ostream
//		converters (Which eliminated many errors).
//
#ifndef _BASICCHANNEL_H_
#define _BASICCHANNEL_H_

//
// include files
//
#include <iostream.h>
#include <fstream.h>
#include "bstring.h"
#include "basicfun.h"

#ifdef RSTS_FILES
const int MaxChannel = 13;		// Channel 12 is maximum
#else
const int MaxChannel = 100;		// Channel 99 is maximum
#endif

//
// Global Variables
//
extern int Recount;

static const int BASIC_FORINPUT = 1;
static const int BASIC_FOROUTPUT = 2;
static const int MODIFY = 1;

class BChannel;

typedef BChannel& (*__bmanip)(BChannel&);

class BChannel
{
private:
	BString filename;		// Name of file open on channel
	ostream* outstream;		// ostream attached to file
	istream* instream;		// istream attached to file
	int ioflag;			// Input/Output flag

public:
	int cpos;			// Cursor position

public:
	//
	// Constructors
	//
	BChannel();
	~BChannel();

	ostream& GetOstream() { return *outstream; }
	istream& GetIstream() { return *instream; }
//	operator ostream&() { return *outstream; }
//	operator istream&() { return *instream; }


	//
	// Open
	//
	int Open();
	int Open(const BString& Name);
	int Open(const char* Name);
	int ForInput();
	int ForOutput();

	int OpenZero();

	//
	// RMS Access
	//
	int Close();
	int Delete();
	int Find();
	int Get();
	int Put();
	int Reset();
	int Unlock();

	//
	// Inputline
	//
	int getline(char* Output, int MaxChar);
	BString Tab(int x);

	//
	// Special
	//
	char* BufferLoc();
	void SetRecordSize();
	void SetFileSize();
	void SetExtendSize();
	void SetWindowSize();
	void SetBlockSize();
	void SetBucketSize();
	void SetMode();
	void SetBuffer();
	void SetDefaultName();
	void SetKey();
	void SetTemporary();
	void SetContiguous();
	void SetRewind();
	void SetSpan();
	void SetOrginization();
	void SetAccess();
	void SetAllow(const int);
	void SetConnect();
	void SetMap();
	void SetRecordType();
	void GoofyOpenStuff();
	void SetRecord(int);
	int Ccpos();

	//
	// Output functions (Horrible Hack, needs lots of work)
	//
	friend BChannel& operator<< (BChannel& os, const BString& bs);
	friend BChannel& operator<< (BChannel& os, const char* bs);
	friend BChannel& operator<< (BChannel& os, double bs);
	friend BChannel& operator<< (BChannel& os, Int32 bs);
	friend BChannel& operator<< (BChannel& os, Int bs);
	friend BChannel& operator<< (BChannel& os, char bs);

	//
	// This assumes that __omanip is defined already in iostream.h
	//
	BChannel& operator<<(__bmanip func) { return (*func)(*this); }

	//
	// Input functions (Horrible Hack, needs lots of work)
	//
	friend BChannel& operator>> (BChannel& is, BString& bs);
	friend BChannel& operator>> (BChannel& is, double& bs);
	friend BChannel& operator>> (BChannel& is, Int32& bs);
	friend BChannel& operator>> (BChannel& is, Int& bs);
	friend BChannel& operator>> (BChannel& is, char& bs);

};

//
// Manipulators
//
extern BChannel& endl(BChannel& outs);

//
// Define the array of channels
//
extern BChannel BasicChannel[MaxChannel + 1];
inline int Ccpos(int x) { return BasicChannel[x].Ccpos(); }

#endif
