// basicfun.h
//
//	Generic basic functions
//
// Description:
//
//	Header file for special routines written to handle basic
//	functions that do not have a direct C++ equivelent.
//
// History:
//
//	09/20/1996 - Kevin Handy
//		Put '#ifdef' around entire include file.
//
//	04/09/1997 - Kevin Handy
//		Move stuff not related to strings from bstring.h into here.
//		Move channel stuff to basicchannel.h
//
//	04/11/1997 - Kevin Handy
//		Start coding some "ON ERROR" stuff.
//
//	12/04/1998 - Kevin Handy
//		Make floatrand always return number [0..1]
//
//	12/06/1998 - Kevin Handy
//		Make Tine inline, so I done get so bloody many link
//		errors.
//
#ifndef _BASICFUN_H_
#define _BASICFUN_H_

#include <stdlib.h>
#include <math.h>
#include <setjmp.h>
#include <time.h>

#include "bstring.h"

//
// Define some variable types
//
typedef int Int;			// Whatever
typedef short int Int16;		// 16 bit integer
typedef long int Int32;			// 32 bit integer

typedef double Dfloat;			// double float type
typedef double Gfloat;			// G float type
typedef double Hfloat;			// H float type
typedef float Sfloat;			// single float type

//
// Hard coded constants
//
static const char BEL = 7;		// Bell
static const char BS = 8;		// Backspace
static const char HT = 9;		// Horizontal tab
static const char LF = 10;		// Line feed
static const char VT = 11;		// Vertical tab
static const char FF = 12;		// Form feed
static const char CR = 13;		// Carriage return
static const char SO = 14;		// Shift out
static const char SI = 15;		// Shift in
static const char ESC = 27;		// Escape
static const char SP = 32;		// Space
static const char DEL = 127;		// Delete
#ifndef PI
static const double PI = 3.1415926535;// Pi (Someone else already defined this)
#endif

//
// Error routines
//
extern long ErrNum;			// Error Number
extern long ErrLine;			// Line Number
extern long Status;			// Status flag

//
// Horrible gosub/return code, but it does work (Icky icky icky)
//
#define BStack(x) jmp_buf BGoStack[x]; int BGoCount = 0;
#define BGosub(x) if (setjmp(BGoStack[BGoCount++]) == 0) { goto x; }
#define BReturn longjmp(BGoStack[--BGoCount], 1);

//
// Now for some nasty on-error-goto code.
//
//	It's not completely compatible with the "On error goto"
//	code, but it's better than nothing.
//
#define OnErrorDie() cerr << "%Error " << ErrNum << " at " << ErrLine << endl; exit(EXIT_FAILURE);
#define OnErrorZero if (setjmp(ErrorStack) == 0) { OnErrorDie(); }
#define OnErrorStack jmp_buf ErrorStack;
#define OnErrorGoto(x) if (setjmp(ErrorStack) != 0) { goto x; }
#define OnResumeLine(x) goto x;
#define OnErrorHit(x) ErrNum = x; longjmp(ErrorStack);

//
// Rnd (random number between 0 and 1)
//	This is a real nasty hack, and may not produce as
//	wide a range of random numbers as basic desires.
//
inline double floatrand(double fmax = 1)
{
	return ((double) rand()) / ((double) RAND_MAX); // * fmax;
}

//
// Some simple math formulas
//
//inline double sqrt(double x) { return pow(x, 0.5); }
inline int sqrt(int x) { return (int)pow((double)x, 0.5); }
inline long sqrt(long x) { return (long)pow((double)x, 0.5); }
inline int sgn(double x) { if (x > 0) return 1; else if (x < 0) return -1; else return 0; }
inline int sgn(int x) { if (x > 0) return 1; else if (x < 0) return -1; else return 0; }
inline int sgn(long x) { if (x > 0) return 1; else if (x < 0) return -1; else return 0; }
inline long Time(int code) { return time(NULL) % 86400; }
inline int Swap(int x) { return ((x & 0xff) << 8) | ((x >> 8) & 0xff); }

//
// Error trapping stuff
//
class BasicError
{
public:
	int ErrorNo;
public:
	BasicError() { ErrorNo = 0; }
	BasicError(int x) { ErrorNo = x; ErrNum = x; }
};

BString sys(const BString& Source);
BString Qdate(int x);
BString Qtime(int x);

#endif
