//
// pusing.h
//
//	Header file for print using class.
//
// History
//
//	04/26/1997 - Kevin Handy
//		Initial version
//
//	03/18/2000 - Kevin Handy
//		Added SetFormat(BString)
//

class PUsing
{
private:
	const char* BaseFormat;		// Pointer to original format string
	int BaseLength;			// Length of format string (Mat not be
					//   null terminated)
	int BasePtr;			// Current pointer into format
	char* Outdata;			// Pointer to output string being built
	char* OutdataPtr;		// Pointer to end of Output

public:
	PUsing();			// Constructor
	PUsing(const char* Format);	// Constructor with a base format
	PUsing(const BString& Format);	// Constructor with a base format

	void SetFormat(const char* Format)
		{ BaseFormat = Format; BaseLength = strlen(Format);
		BasePtr = 0; }
	void SetFormat(const BString& Format)
		{ BaseFormat = Format.c_str(); BaseLength = Format.length();
		BasePtr = 0; }

	//
	// Various output commands
	//
	BString Output(const char* Value, int ValueLength);
	inline BString Output(const char* Value)
		{ return Output(Value, strlen(Value)); }
	BString Output(const BString& Value);
	BString Output(const double Value);
	BString Output(const int Value);
	BString Output(const long Value);

	BString Finish();		// Final characters in format string

private:
	int SkipFront();
	void ScanStringFormat(int &FWidth, int &FSide);
	void FormatString(const char* ParmPtr, int ParmLength,
		int Length, int Side);
};

//
// Variations of the "FORMAT$(value, format)" command
//
inline BString Format(const BString& Value, const BString& Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const BString& Value, const char* Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const char* Value, const char* Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const char* Value, const BString& Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const double Value, const BString& Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const double Value, const char* Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const int Value, const BString& Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }
inline BString Format(const int Value, const char* Format)
	{ PUsing Puse(Format); BString One = Puse.Output(Value);
	BString Two = Puse.Finish(); return One + Two; }

