/*******************************************************************************

Program:

	Basic.h
	01.01.00

Title:

	Basic Translater Main Header File

Description:

	This is the main header file for the basic translater.

Bugs:

History:

	10/22/1996 - Kevin Handy
		Added variables 'IntegerType' and 'RealType'.

	08/09/1999 - Kevin Handy
		Start of table to convert strings to node types
		'node_type[]' so that we can greatly reduce the
		complexity of the lexer.

*******************************************************************************/
#ifndef _BASIC_H_
#define _BASIC_H_
#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

//
// Optional stuff in compile
//
#define USE_LONGOPT	/* Use getopt_long instead of getopt */

//
// Include files
//
#include <fstream.h>
#include <string>
#include <map>

//
// These functions are refered to by the local include files
//
void UpperCase(string&);
void LowerCase(string&);

//
// Local include files
//
class VariableList;
class Node;

//
// Format for transfering info between flex and yacc
//
#define YYSTYPE Node *

//
// Required by Lex and Yacc
//
extern FILE *yyin;
extern int xline;
extern int yydebug;

//
// External stuff in main.c
//
extern VariableList *Variables;
extern map<string, int> node_type;

extern int CompileFlag;

extern int KeepAllLines;
extern int NeedIostreamH;
extern int NeedStdlibH;
extern int NeedStringH;
extern int NeedMathH;
extern int NeedErrorH;
extern int NeedRFA;
extern int NeedBasicFun;
extern int NeedTimeH;
extern int NeedDataList;
extern int NeedPuse;
extern int NeedChannel;
extern int NeedUnistd;
extern int NeedVirtual;

extern int IntegerType;
extern int RealType;
extern int DefaultType;

extern int DebugDump;
extern int DebugDumpOne;
extern int DebugInput;
extern int VariableDump;
extern int PositionDump;

extern ostream* OutFile;

void DoProgram(Node *Program);

//
// Parsing stuff
//
extern int xline;
extern int include_stack_pointer;
extern Node* CommentList;
extern char* include_name[];

int yyparse();
int yyerror(char *s);
int yylex( void );

#ifdef __MSDOS__
extern "C"
{
	void* alloca(int);
}
#endif

#endif
