//
// Nodes.h
//
//	This file contains the definition for the nodes that hold
//	the tokenized program.
//
//	The main way to think of a node is in three parts
//	1. What this node is (Start of an IF statement)
//	2. What are the patameters 'Tree' (the test as in 'if xxx')
//	3. What blocks follow 'Block' (fall-through, then, else)
//
#ifndef _NODES_H_
#define _NODES_H_

#include <iostream.h>
#include <string>

//
// Forward definitions
//
class VariableStruct;

#define NNULL 0

//
// Main structure defining the program
//
class Node
{
private:
	int Type;		// Node Type
	string TextValue;	// Text value associated with node
	Node *Tree[5];		// Pointers to parameters
	Node *Block[3];		// Pointers to code blocks
	static int Level;	// Indentation level
	int FromInclude;	// Is this from an include file?
	int lineno;		// Source line for error messages

public:
	Node(int TypeValue = 0, string xTextValue = "", int Include = 0, int Line = 0)
	{ Type = TypeValue; TextValue = xTextValue;
	  Tree[0] = NNULL; Tree[1] = NNULL; Tree[2] = NNULL;
	  Tree[3] = NNULL; Tree[4] = NNULL; Block[0] = NNULL;
	  Block[1] = NNULL; Block[2] = NNULL;
	  FromInclude = Include; lineno = Line; }
	Node(const Node &OldNode);
	~Node() { delete Tree[0]; delete Tree[1]; delete Tree[2]; delete Tree[3];
		delete Tree[4]; delete Block[0]; delete Block[1]; }

	Node* GetDown(int Ptr = 0) { return Block[Ptr]; }
	int GetType() { return Type; }
	string GetText() { return TextValue; }
	Node* GetTree(int Ptr) { return Tree[Ptr]; }
	string NoParen();

	void SetType(int NewType) { Type = NewType; }
	void SetInclude(int NewValue) { FromInclude = NewValue; }

	Node *Link(
		struct Node *Node0 = NNULL,
		struct Node *Node1 = NNULL,
		struct Node *Node2 = NNULL,
		struct Node *Node3 = NNULL,
		struct Node *Node4 = NNULL);
				// Attach parameter nodes to main node
	Node *DownLink(struct Node *Node0 = NNULL,
		int Ptr = 0);
				// Attach links at same level
	void UnDownLink(int Ptr = 0) { Block[Ptr] = NNULL; }
	void UnLink(int Ptr) { Tree[Ptr] = NNULL; }

	void PrintTree(const string &Level = "", int flag = 0, char Node = '*');
	void VariableScan(int AFlag);
	void VariableScanOne(int AFlag);
	int ScanType();
	void Output(ostream& os);
	void OutputCode(ostream& os);
	void OutputCodeOne(ostream& os);
	string Expression();

	int IsLogical();
	int IsSimpleInteger();
	int CountParam();
	void StripLeadZero()
#ifdef STLREMOVE
	{ while(TextValue[0] == '0') TextValue.remove(0, 1); }
#else
	{ while(TextValue[0] == '0') TextValue.erase(0, 1); }
#endif
	void SmoothTypes(Node *DefaultType = NULL);

private:
	void OutputBlock(ostream& os);
	void OutputPrint(ostream& os);
	void OutputIPChannel(ostream& os, int InputFlag);
	void OutputInput(ostream& os, int InputFlag);
	int OutputPrintData(ostream& os, int InputFlag);
	int CheckCpos();
	int OutputInputData(ostream& os, int InputFlag);
	void OutputData(ostream& os);
	void OutputDataValue(ostream& os);
	void OutputMap(ostream& os);
	void ScanMap();
	void ScanOneMap(VariableStruct* MapVar);
	void OutputCaseLabel(ostream& os);
	string OutputVarName(Node* array, int indefine = 0);
	string Indent() { return string(Level >= 0 ? Level : 0, '\t'); }
	void ScanVarList(int ThisType, int ThisClass, int Status);
	void OutputRemark(ostream& os);
	string OutputLabel();
	int OutputOngo(ostream& os, const char* GoType, int GoLevel);
	void OutputDefinitionList(ostream& os, Node* MainType,
		int ExtType, int GlobalStat = 0);
	void Node::OutputVirtualList(ostream& os, Node* Channel,
		Node* MainType, int ExtType, int GlobalStat = 0);
	string OutputDefinitionParams(Node* MainType,
		int ExtType, int GlobalStat = 0);
	string OutputDefinition(Node* MainType, int ExtType);
	string OutputNewDefinition();
	string OutputArrayParam(int Increment);
	void OutputOpenStuff(ostream& os, Node* Channel);
	void OutputClose(ostream& os);
	string OutputBstringText();
	string OutputInteger();
	void OutputField(ostream& os);
	int IsReallyString(void);
	void OutputGetPutOptions(Node* Channel, ostream& os);
	int GetNodeVarType();
	string OutputForcedType(int VarType);
	void OutputPrototypes(ostream& os);
	void ScanForDefstar(int);
	void ScanOneDefstar(int);
	string OutputPassmech(int FunctionFlag);
	string OutputNodeVarType();
};

#endif
