/*******************************************************************************

Program:

	nodes1.cc

Title:

	Routines to work with nodes (generic routines).

Description:

	This file contains various routines for manipulating the tree
	nodes used when parsing a basic program.

	This file contains the generic functions.

Compile:

	cc [noopt]

History:

	07/08/1999 - Kevin Handy
		Split nodes.cc into three sections

	08/20/1999 - Kevin Handy
		Modify PrintTree to add cases of Tree[3] and Tree[4].

*******************************************************************************/

//
// System Include Files
//
#include <iostream.h>
#include <stdio.h>
#include <assert.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

//
// Project Include Files
//
#include "basic.h"
#include "parse.h"
#include "nodes.h"
#include "varlist.h"

//
// Module Variables
//
int GosubFlag = 0;
int OnErrorFlag = 0;

int Node::Level = 0;

/*******************************************************************************


	Node(Node &)

Title:

	Create a new node just like an existing node

Description:

	Creates a new node which is a copy of an existing node,
	including all linked nodes.

Bugs:

*******************************************************************************/

Node::Node(const Node &OldNode)
{
	int loop;			// Generic loop variable

	//
	// Copy over all the easy bits
	//
	Type = OldNode.Type;
	TextValue = OldNode.TextValue;
	Level = OldNode.Level;
	FromInclude = OldNode.FromInclude;
	lineno = OldNode.lineno;

	//
	// Copy over tree entries
	//
	for (loop = 0; loop < 5; loop++)
	{
		if (OldNode.Tree[loop] == NNULL)
		{
			Tree[loop] = NNULL;
		}
		else
		{
			//
			// Recursive call here
			//
			Tree[loop] = new Node(*OldNode.Tree[loop]);
		}
	}

	//
	// Copy over block entries
	//
	//	Should we really copy over the block entries?
	//	This may proove to be rather a lot of stuff.
	//
	for (loop = 0; loop < 3; loop++)
	{
		if (OldNode.Block[loop] == NNULL)
		{
			Block[loop] = NNULL;
		}
		else
		{
			//
			// Recursive call here
			//
			Block[loop] = new Node(*OldNode.Block[loop]);
		}
	}
}

/*******************************************************************************


	Link()

Title:

	Link child nodes into parent node.

Description:

	This function will link several child nodes into a parent node.

Return Value:

	Returns a pointer to the parent node (passed as the first
	parameter)

Bugs:

*******************************************************************************/

Node *Node::Link
(
	Node *Node0,
	Node *Node1,
	Node *Node2,
	Node *Node3,
	Node *Node4
)
{
	//
	// Trap stupid bugs
	//
#if 0
	//
	// Debugging code
	//
if (this == NNULL)
{
 cerr << "this == 0 Error: ";
 if (Node0 != NNULL)
 {
  cerr << " 0 " << Node0->Type;
 }
 if (Node1 != NNULL)
 {
  cerr << " 1 " << Node1->Type;
 }
 if (Node2 != NNULL)
 {
  cerr << " 2 " << Node2->Type;
 }
 if (Node3 != NNULL)
 {
  cerr << " 3 " << Node3->Type;
 }
 if (Node4 != NNULL)
 {
  cerr << " 4 " << Node4->Type;
 }
 cerr << endl;
}
#endif
	assert(this != NNULL);

	//
	// Attach them together
	//
	if (Node0 != NNULL)
	{
#if 0
		//
		// Debugging code
		//
		if (Tree[0] != NNULL)
		{
		cerr << "Error Tree0 != 0: " << Type << " " << TextValue << endl <<
		 ", append: " << Tree[0]->Type << " " << Tree[0]->TextValue << endl;
		cerr << "Tree[0] = "; Tree[0]->PrintTree();
		cerr << "This = "; PrintTree();
		}
#endif
		assert(Tree[0] == NNULL);
		assert(Node0 != this);
		Tree[0] = Node0;
	}
	if (Node1 != NNULL)
	{
		assert(Tree[1] == NNULL);
		assert(Node1 != this);
		Tree[1] = Node1;
	}
	if (Node2 != NNULL)
	{
		assert(Tree[2] == NNULL);
		assert(Node2 != this);
		Tree[2] = Node2;
	}
	if (Node3 != NNULL)
	{
		assert(Tree[3] == NNULL);
		assert(Node3 != this);
		Tree[3] = Node3;
	}
	if (Node4 != NNULL)
	{
		assert(Tree[4] == NNULL);
		assert(Node4 != this);
		Tree[4] = Node4;
	}

	//
	// Return main node back
	//
	return(this);
}

/*******************************************************************************

Member Function:

	DownLink()

Title:

	Link two nodes together at same level.

Description:

	This function will link two parent nodes together at the same
	tree level. If one or the other node doesn't exist, it will
	make whichever node exists the main node.

Return Value:

	Returns the top node.
Bugs:

*******************************************************************************/

//
// Link several nodes together through the Nodes
//
Node *Node::DownLink
(
	Node *Node0,
	int Ptr
)
{
	//
	// Don't link if nothing to link to
	//
	if (this == NNULL)
	{
		return(Node0);
	}

	//
	// Skip out if link node is NNULL
	//
	if (Node0 == NNULL)
	{
		return this;
	}

	//
	// Test for stupid bugs
	//
	assert(Node0 != this);

	//
	// If we aren't at the top of the stack, scan to the end
	//
	if (Block[Ptr] == NNULL)
	{
		Block[Ptr] = Node0;
		return this;
	}

	Node* LookDown = Block[Ptr];
	while (LookDown->Block[0] != NNULL)
	{
		LookDown = LookDown->Block[0];
	}

	//
	// Attach them together
	//
	LookDown->Block[0] = Node0;

	//
	// Return main node back
	//
	return(this);
}

/*******************************************************************************

Global Function:

	PrintTree()

Title:

	Prints the node tree.

Description:

	This function will print out a simple picture of the node
	and its relationship to it's children.

Bugs:

	Only displays to stdout. Ugly display.

*******************************************************************************/

//
// Print out tree in a tree structure
//
void Node::PrintTree(const string &Level, int Flag, char Node)
{
	//
	// Not really a node
	//
	assert(this != NNULL);

	//
	// Local variables
	//
	string DownLevel;

	//
	// Make sure we have an active node
	//
	if ((Block[0] != NNULL) || (Block[1] != NNULL) ||
		(Block[2] != NNULL) || Flag)
	{
		DownLevel = " | ";
	}
	else
	{
		DownLevel = "   ";
	}

	string NextLevel = Level + DownLevel;

	//
	// Print bars for this level
	//
	cout << Level << " " << Node << "-" << Type;

	if (TextValue.length() != 0)
	{
		cout << " Text: " << TextValue;
	}

	//
	// Output variable type
	//
	cout << " <" << DumpVarType(GetNodeVarType()) << ">";

	cout << endl;

	//
	// Print Parameter Levels
	//
	for (int loop = 0; loop < 5; loop++)
	{
		if (Tree[loop] != NNULL)
		{
			int fflag = 0;
			for (int inloop = loop + 1; inloop < 5; inloop++)
			{
				if (Tree[inloop] != NNULL)
				{
					fflag = 1;
				}
			}
			Tree[loop]->PrintTree(NextLevel, fflag, loop + '0');
		}
	}

	//
	// Link down to next parent if possible
	//
	if (Block[1] != NNULL)
	{
		Block[1]->PrintTree(NextLevel, 0, 'T');
	}
	if (Block[2] != NNULL)
	{
		Block[2]->PrintTree(NextLevel, 0, 'E');
	}
	if (Block[0] != NNULL)
	{
		Block[0]->PrintTree(Level, 0, 'D');
	}
}

/***********************************************************************
 * GetNodeVarType
 *
 *	Calculate the varible type for a given node.
 *
 * Description:
 *
 *	Determines the variable type for a given node, and returns
 *	the specified type.
 */
int Node::GetNodeVarType()
{
	VariableStruct *ThisVar;

	//
	// Handle this node type
	//
	switch (Type)
	{
	//
	// Takes the type of the rvalue
	//
	case '#':
	case '(':
	case BAS_N_ASSIGN:
	case BAS_N_ASSIGNLIST:
	case BAS_S_BY:
	case BAS_N_FORASSIGN:
	case BAS_N_UMINUS:
	case BAS_N_UPLUS:
		if (Tree[0] != NNULL)
		{
			return Tree[0]->GetNodeVarType();
		}
		break;

	//
	// Takes the 'higher' of the two types
	//
	case '+':
	case '-':
	case '*':
	case '/':
	case '^':
		assert(Tree[0] != NNULL);
		assert(Tree[1] != NNULL);
		return ChooseVarType(Tree[0]->GetNodeVarType(),
			Tree[1]->GetNodeVarType());

	case BAS_N_LIST:
		if ((Tree[0] != NNULL) && (Tree[1] != NNULL))
		{
			return ChooseVarType(Tree[0]->GetNodeVarType(),
				Tree[1]->GetNodeVarType());
		}
		break;
	//
	// Always an integer
	//
	case '=':
	case '>':
	case '<':
	case BAS_X_LAND:
	case BAS_S_AND:
	case BAS_S_EQV:
	case BAS_S_IMP:
	case BAS_S_ERL:
	case BAS_X_GE:
	case BAS_V_INT:
	case BAS_V_INTEGER:
	case BAS_X_LE:
	case BAS_X_NEQ:
	case BAS_S_NOT:
	case BAS_X_LNOT:
	case BAS_X_LOR:
	case BAS_S_OR:
	case BAS_X_UNARYGT:
	case BAS_X_UNARYGE:
	case BAS_X_UNARYLE:
	case BAS_X_UNARYLT:
	case BAS_X_UNARYNEQ:
	case BAS_S_XOR:
		return VARTYPE_INTEGER;

	//
	// Always a floating point
	//
	case BAS_V_FLOAT:
		return VARTYPE_REAL;

	//
	// Always a string
	//
	case BAS_V_TEXTSTRING:
		return VARTYPE_DYNSTR;

	//
	// Function references
	//
	case BAS_V_FUNCTION:
	case BAS_V_NAME:
		//
		// Look for variable in table
		//
		ThisVar = Variables->Lookup(TextValue, Tree[0]);

		//
		// Print this variable
		//
		if (ThisVar != VNULL)
		{
			return ThisVar->GetType();
		}
		break;

	//
	// Gonna havta handle these at some time, but for now ...
	//
	case BAS_V_PREDEF:
	case BAS_X_STRREF:
	case BAS_N_STRUCTNAME:
	case BAS_N_TYPEFUN:
	case BAS_N_VARTYPE:
	case BAS_S_BYTE:
	case BAS_S_DOUBLE:
	case BAS_S_GFLOAT:
	case BAS_S_HFLOAT:
	case BAS_S_LONG:
	case BAS_S_WORD:
	case BAS_S_INTEGER:
	case BAS_S_REAL:
	case BAS_S_RFA:
	case BAS_S_STRING:
	case BAS_S_DECIMAL:

		break;
	}

	//
	// I don't know!
	//
	return VARTYPE_NONE;
}

/***********************************************************************
 * Member Function:
 *
 *	Node::IsReallyString
 *
 * Title:
 *
 *	Determines if a text string is really a simple string.
 *
 * Description:
 *
 *	Sometines a TEXTSTRING is actually a number (ie "15"C), and
 *	this function is used when a determination is necessary
 *	about the "really a string" quality is needed.
 */
int Node::IsReallyString(void)
{
	if (Type == BAS_V_TEXTSTRING)
	{
		if (*(TextValue.end() - 1) == '"')
		{
			return 1;
		}
	}
	return 0;
}

/***********************************************************************
 * IsLogical
 *
 *	Is the node a Logical operation (Returns true/false indication)
 *
 * Description:
 *
 *	This function is used to determine if a nodes is a logical
 *	operation (returning true/faslse), so that higher code can
 *	optimize itself to also be logical.
 */
int Node::IsLogical()
{
	switch(Type)
	{
	case '<':
	case '>':
	case BAS_X_NEQ:
	case BAS_X_GE:
	case BAS_X_LE:
	case '=':
	case BAS_X_LNOT:
	case BAS_X_LAND:
	case BAS_X_LOR:
		//
		// logical operations
		//
		return 1;

	case '(':
		//
		// Look beneath the skin
		//
		return Tree[0]->IsLogical();

	default:
		//
		// By default, things are not logical
		//
		return 0;
	}
}

/***********************************************************************
 * IsSimpleInteger
 *
 *	Is the node a simple integer value
 *
 * Description:
 *
 *	This function is used to determine if the node is a simple
 *	integer value (i.e. one which we can do a 'atoi' on and get
 *	correct results).
 *
 *	Used mostly for the array size definition routines when trying
 *	to increase the dimension sizes by one.
 */
int Node::IsSimpleInteger()
{
	//
	// Base type must be integer
	//
	if ((Type == BAS_V_INTEGER) || (Type == BAS_V_INT))
	{
		//
		// Has to start with a digit
		//
		if (isdigit(TextValue[0]))
		{
			//
			// has to end with a digit or a '%'
			//
			char LastChar = *(TextValue.end() - 1);
			if (isdigit(LastChar) || (LastChar == '%'))
			{
				return 1;
			}
		}
	}

	return 0;
}


/***********************************************************************
 * CountParam
 *
 *	Count the number of parameters for a function/array
 *
 * Description:
 *
 *	This function is used to determine the number of parameters
 *	that a function/array uses.
 */
int Node::CountParam()
{
	int Counter;

	switch(Type)
	{
	case BAS_N_LIST:
		if (Tree[0] != NNULL)
		{
			Counter = Tree[0]->CountParam();
		}
		else
		{
			Counter = 1;		// Empty parameter
		}

		if (Tree[1] != NNULL)
		{
			Counter += Tree[1]->CountParam();
		}
		else
		{
			Counter++;		// Empty parameter
		}
		break;

	default:
		//
		// By default, things are not logical
		//
		Counter = 1;
	}
	return Counter;
}

