/*******************************************************************************

Program:

	Variable.h
	01

Title:

	Header file for variable table

Description:

	This is the header file for the variable table/

Bugs:

	Lousy. Needs complete re-write

History:

	01.01.02 - 10/22/1996 - Kevin Handy
		Added 'SINGLE', "GFLOAT', HFLOAT' types.

	01.03.01 - 04/19/1997 - Kevin Handy
		Start working on parameters, pass mechanism
		Make use of more C++'isms.

	01.04.01 - 07/06/1997 - Kevin Handy
		Renumber VARTYPE's so they fall in weight order.
		Allows easier determination of conversions.

	01.04.02 - 11/28/1997 - Kevin Handy
		Modified to use STL classes instead of my own
		(slow) lists.

	01.05.01 - 12/15/1997 - Kevin Handy
		Changed 'Node' member 'TextValue' from 'char*' to 'string'

	01.05.02 - 02/15/1998 - Kevin Handy
		Changed VariableList to use a <vector> instead of
		the two variables "Global" and "Local", in preparation
		of handling common variables in "DEF*" statements.

		Moved three functions from VariableList to
		ListOfVariables, where they really belonged,
		and lost the 'Tree' on the end of their names.

	01.05.03 - 02/15/1998 - Kevin Handy
		Made VariableList inherit BaseVariableList instead
		haveing a variable based on that type. Get to lose a
		lot of references to 'Variables' that way, as well as
		a lot of junk trying to make iterators and reverse
		iterators happy.

	01.05.04 - 09/04/1998 - Kevin Handy
		Split out the headers for varlist.h from this code

*******************************************************************************/

//
// Variable.h - Some stuff just in case if isn't defined anywhere else
//
#ifndef _VARIABLE_H_
#define _VARIABLE_H_

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

//
// Include files
//
#include <iostream.h>
#include <string>
#include <map>
#include <vector>
#include "nodes.h"
#include "parse.h"

#define VNULL (VariableStruct*)NULL

//
// Variable Type Definitions
//
//	These are numbered according to the 'weight' of the
//	variable type. (With NONE being the lowest of the low)
//
static const int VARTYPE_NONE =		0;	// Undefined
static const int VARTYPE_VOID =		1;	// Void (no) return
static const int VARTYPE_STRUCT =	2;	// Structure
static const int VARTYPE_RFA =		3;	// RFA
static const int VARTYPE_LABEL =	4;	// Label
static const int VARTYPE_FIXSTR =	5;	// Fixed length string
static const int VARTYPE_DYNSTR =	6;	// Dynamic String
static const int VARTYPE_BYTE =		7;	// Byte (char)
static const int VARTYPE_WORD =		8;	// 16 bit integer
static const int VARTYPE_INTEGER =	9;	// 16 bit integer
static const int VARTYPE_LONG =		10;	// 32 bit integer
static const int VARTYPE_SINGLE =	11;	// Single precision float
static const int VARTYPE_REAL =		12;	// float type
static const int VARTYPE_DOUBLE =	13;	// double type
static const int VARTYPE_HFLOAT =	14;	// HFloat floating point
static const int VARTYPE_GFLOAT =	15;	// GFloat floating point
static const int VARTYPE_DECIMAL =	16;	// Label

static const int VARCLASS_NONE =	0;	// Regular variable
static const int VARCLASS_ARRAY =	1;	// An Array
static const int VARCLASS_FUNC =	2;	// Function
static const int VARCLASS_CONST =	3;	// Constant
static const int VARCLASS_LABEL =	4;	// Label
static const int VARCLASS_MAP =		5;	// Map variable

static const int VARMECH_NONE =		0;	// Whatever goes
static const int VARMECH_REF =		1;	// By Reference
static const int VARMECH_DESC =		2;	// By Descriptor
static const int VARMECH_VALUE =	3;	// By Value
static const int VARMECH_XREF =		4;	// Use C++ '&' reference mechanism (??)

//
// Structure defining variables
//
class VariableStruct
{
public:
	string BasicName;		// Name used in BASIC
	int Type;			// Type (integer, real, ...)
	int Class;			// Variable Class (function, array, ...)
	string CName;			// Name to be used in C
	int Output;			// Already output to source?
	int EverUsed;			// Is it ever used (Label)
	int ParCount;			// Parameter count
	int GlobalFlag;			// Global/Local flag

public:
	VariableStruct(const string VarName = "", int newType = 0,
		int newClass = 0, int newOutput = FALSE, int newParCount = 0)
	{ Type = newType; Class = newClass; Output = newOutput;
	  ParCount = newParCount; EverUsed = 0;
	  BasicName = VarName; UpperCase(BasicName); GlobalFlag = 0;
	  assert(Type != 13 || isdigit(VarName.c_str()[0]) == 0); }
	VariableStruct(const VariableStruct &OldStruct)
	{ Type = OldStruct.Type; Class = OldStruct.Class;
	  Output = OldStruct.Output; ParCount = OldStruct.ParCount;
	  EverUsed = OldStruct.EverUsed; BasicName = OldStruct.BasicName;
	  CName = OldStruct.CName; GlobalFlag = OldStruct.GlobalFlag; }
//	~VariableStruct() {};

	void Dump() const;
	string OutputDef();
	string GetName() { if (CName.length() != 0 ) return CName;
		else return BasicName; }
	void SetCName(const char* NewName) { CName = NewName; }
	int GetType() { return Type; }
	void SetType(int tType) { Type = tType; }
	void SetClass(const int NewClass) { Class = NewClass; }
	int GetClass() { return Class; }
	void IncrUsed() { EverUsed++; }
	int GetEverUsed() { return EverUsed; }

	VariableStruct& operator = (const VariableStruct &OldStruct)
	{ Type = OldStruct.Type; Class = OldStruct.Class;
	  Output = OldStruct.Output; ParCount = OldStruct.ParCount;
	  EverUsed = OldStruct.EverUsed; BasicName = OldStruct.BasicName;
	  CName = OldStruct.CName; return *this; }

};


//
// Functions related to variables
//
const char* DumpVarType(int VarType);
string OutputVarType(int VarType);
int GuessVarType(const string Name);
int UnguessVarType(int vartype);

//
// Which type to raise expression to
//
static inline int ChooseVarType(int a, int b) { return (a > b) ? a : b; }

#endif
