/*******************************************************************************

Program:

	varlist.cc
	01.04

Title:

	Routines to handle variables.

Description:

	This file contains routines to handle the variable definitions
	necessary for basic.

Bugs:

History:

	08/30/1996 - Kevin Handy
		Update version to 1.1, and install on internet.

	10/09/1996 - Kevin Handy
		Changes "ascii" from "(char)(x)" to "Ascii(x)".

	10/20/1996 - Kevin Handy
		Output 'sqrt' instead of 'sqr' for square root.
		Added several more functions to table.

	01/01/1997 - Kevin Handy
		Output Qstring when STRING$() is callewd, because of
		problems with existing c++ functions.
		Also Qspace for space.

	01/02/1997 - Kevin Handy
		Output "trm" instead of "trim" for the trim function

	01/05/1997 - Kevin Handy
		Handle num$ and num1$ as Qnum and Qnum1.

	01/06/1997 - Kevin Handy
		Handle rad$ as Qrad.

	01/26/1997 - Kevin Handy
		Set up Qdate, Qtime, sys.

	01/29/1997 - Kevin Handy
		Set up INSTR, SEG$.

	02/07/1997 - Kevin Handy
		Set up STR$(), ERT$() functions.

	04/06/1997 - Kevin Handy
		Handle FNxxx type functions better.

	04/21/1997 - Kevin Handy
		Add several new functions to function table.

	06/16/1997 - Kevin Handy
		Set version to 1.4

	06/20/1997 - Kevin Handy
		Add 'argc' and 'argv' to reserved words list.

	07/07/1997 - Kevin Handy
		Add 'inline' to list of reserved words.

	08/05/1997 - Kevin Handy
		Do fixup's on global functions as well as local ones.

	11/28/1997 - Kevin Handy
		Modifications necessary to use STL functions.

	11/29/1997 - Kevin Handy
		Simplify the BASIC to C name conversions so it is more
		of a first-come-first-serve basis instead of the more
		complicated three step process I was using.

	12/15/1997 - Kevin Handy
		Changed 'Node' member 'TextValue' from 'char*' to 'string'

	01/26/1998 - Kevin Handy
		Fixed bug in lookup of FNxxx variables without parameters.

	02/05/1998 - Kevin Handy
		Moved several functions into the header to improve code
		size and speed.

	02/06/1998 - Kevin Handy
		Cleaned up several functions (cruft left over from conversion
		to STL and strings).

	09/04/1998 - Kevin Handy
		Moved some of the inline functions from the header into
		here because the cause the VAX/GAS to crash with a
		horrible crash (NewLevel, KillLevel)

	12/05/1998 - Kevin Handy
		Modified Lookup to have an option to only look at the
		most current definitions (to fix a DEF problem)
		Modified Define to use the 3rd parm as a hard define flag also.

	12/23/1998 - Kevin Handy
		Added INKEY$ function.

	07/01/2000 - Kevin Handy
		Lose ~VariableList, which was empty.
		Simplify VariableList and NewLevel functions.

*******************************************************************************/

/*
 * System Include Files
 */
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>

//
// Project Include Files
//
#include "basic.h"
#include "varlist.h"
#include "parse.h"


/*******************************************************************************

Member Function:

	VariableList()

Title:

	Constructs a variable list.

Description:

	This routine creates a variable list.

Bugs:

	This function should not be made inline, becaause when that is
	done it takes forever to compile.

*******************************************************************************/

VariableList::VariableList()
{
	insert(begin(), *(new ListOfVariables));
	vcount = 1;

	//
	// Load in hard coded function list
	//
	InitFunctions();
}


void VariableList::NewLevel()
{
	insert(end(), *(new ListOfVariables));
}

void VariableList::KillLevel()
{
	erase(&back());
}

/*******************************************************************************

Member Function:

	VariableList::Lookup()

Title:

	Look for a variable in the table using it's basic name.

Description:

	This routine will scan the variable table for the occurance of a
	specified variable.

Bugs:

*******************************************************************************/

VariableStruct *VariableList::Lookup
(
	string VarName,			// Basic name to look up
	const Node *ArrayRef,		// Array/function flag
	int ScanTopOnly			// Scan Only the Topmost definition
)
{
	ListOfVariables::iterator VarTable;
	string keyname = VarName;
	UpperCase(keyname);

	if ((ArrayRef != NNULL) || (keyname.substr(0,2) == "FN"))
	{
		keyname += "(";
	}

	//
	// Scan the list
	//
	if (ScanTopOnly != 0)
	{
		reverse_iterator loop = rbegin();
		VarTable = (*loop).find(keyname);
		if (VarTable != (*loop).end())
		{
			return &((*VarTable).second);
		}
	}
	else
	{
		for (reverse_iterator loop = rbegin();
			loop != rend(); loop++)
		{
			VarTable = (*loop).find(keyname);
			if (VarTable != (*loop).end())
			{
				return &((*VarTable).second);
			}
		}
	}

	//
	// Not found
	//
	return VNULL;
}

/*******************************************************************************

Mamber Function:

	VariableList::LookupCpp()

Title:

	Look for a variable in the table using the C++ name.

Description:

	This routine will scan the variable table for the occurance of a
	specified variable.

Bugs:

*******************************************************************************/

VariableStruct *VariableList::LookupCpp
(
	const string VarName
)
{
	VariableStruct *VarTable;

	for (reverse_iterator loop = rbegin();
		loop != rend(); loop++)
	{
		VarTable = (*loop).LookupCpp(VarName);

		if (VarTable != VNULL)
		{
			break;
		}
	}

	return VarTable;
}

VariableStruct *ListOfVariables::LookupCpp
(
	const string VarName
)
{
	//
	// Loop through all variables
	//
	for (iterator i = begin(); i != end(); i++)
	{
		//
		// Is this it?
		//
		if (((*i).second.CName.length() != 0) &&
			((*i).second.CName.compare(VarName) == 0))
		{
			return &((*i).second);
		}
	}

	//
	// Didn't find it
	//
	return VNULL;
}

/*******************************************************************************

Mamber Function:

	VariableList::LookupReserved()

Title:

	Look for a variable in the list of reserved names and the
	already used names using the C++ name.

Description:

	This routine will scan the variable table for the occurance of a
	specified variable.

Bugs:

*******************************************************************************/

//
// This list represent names that are not available for one reason or
// another to a C++ program for use as a variable name. It keeps
// generated names from overlapping too badly with existing functions
// and reserved words.
//
// Do not include constants that have been appended to the variable
// list in the startup routines. That would be redundant, and a
// duplication of effort.
//
static const char* ReservedList[] =
{
	"argc",
	"argv",
	"atof",
	"atoi",
	"auto",
	"break",
	"case",
	"char",
	"cin",
	"class",
	"const",
	"continue",
	"cout",
	"default",
	"delete",
	"do",
	"double",
	"else",
	"extern",
	"far",
	"fclose",
	"float",
	"fopen",
	"for",
	"fprintf",
	"free",
	"goto",
	"if",
	"inline",
	"int",
	"long",
	"malloc",
	"namespace",
	"near",
	"new",
	"operator",
	"printf",
	"public",
	"private",
	"register",
	"return",
	"scanf",
	"short",
	"sprintf",
	"static",
	"strcmp",
	"struct",
	"switch",
	"union",
	"using",
	"virtual",
	"void",
	"volatile",
	"template",
	"throw",
	"typedef",
	"unsigned",
	"while",
	0
};

int VariableList::LookupReserved
(
	const char *VarName
)
{
	//
	// Look through reserved word list
	//
	for (int i = 0; ReservedList[i] != 0; i++)
	{
		if (strcmp(ReservedList[i] , VarName) == 0)
		{
			return TRUE;
		}
	}

	//
	// Look through list of already used names
	//
	VariableStruct *VarTable = LookupCpp(VarName);

	if (VarTable != VNULL)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

int VariableList::LookupReserved
(
	const string VarName
)
{
	//
	// Look through reserved word list
	//
	for (int i = 0; ReservedList[i] != 0; i++)
	{
		if (VarName.compare(ReservedList[i]) == 0)
		{
			return TRUE;
		}
	}

	//
	// Look through list of already used names
	//
	VariableStruct *VarTable = LookupCpp(VarName);

	if (VarTable != VNULL)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

/*******************************************************************************

Member Function:

	VariableList::Dump()

Title:

	Print out variable table.

Description:

	Prints out the variables stored in the variable table.

Bugs:

	Ugly output.

*******************************************************************************/

void VariableList::Dump()
{
	int Level = 0;

	for (iterator loop = begin();
		loop != end(); loop++)
	{
		//
		// Dump out global variables
		//
		cout << endl << "// Level " << Level++ << ":" << endl;
		(*loop).Dump();
	}

	cerr << endl;
}

void ListOfVariables::Dump(void)
{
	//
	// Loop through all variables
	//
	for (iterator i = begin(); i != end(); i++)
	{
		//
		// Dump it out
		//
		cout << "//    Key: " << (*i).first << " ";
		(*i).second.Dump();
	}
}


/*******************************************************************************

Member Function:

	VariableList::Define()

Title:

	Define the current node as a variable.

Description:

	This function tries to define the current node as a variable.
	It is used to define those variables that are not defined
	in some other manner by the user.

Bugs:

*******************************************************************************/

void VariableList::Define(Node *ThisNode, int GlobalFlag, int ThisOutput)
{
	//
	// Local Variables
	//
	int Type;
	VariableStruct *ThisVar;
	int ThisClass = VARCLASS_NONE;
	int ParCount = 0;
	int PreDefined = FALSE;
	string TextName = ThisNode->GetText();

	UpperCase(TextName);

	//
	// If the name starts fn..., then it must be a function
	// (it's the law)
	//
	if (TextName.substr(0,2) == "FN")
	{
		ThisNode->SetType(BAS_V_FUNCTION);
		PreDefined = TRUE;
	}

	//
	// Is it already defined as a variable?
	//
	if ((ThisVar = Lookup(TextName, ThisNode->GetTree(0), ThisOutput)) != VNULL)
	{
		//
		// Force the node to be a function if that is what it is.
		//
		if (ThisVar->Class == VARCLASS_FUNC)
		{
			ThisNode->SetType(BAS_V_FUNCTION);
		}
		return;
	}

	//
	// Check the last letter of the variable to determine what type
	// it ought to be.
	//
	Type = GuessVarType(TextName);

	//
	// Is this an array reference?
	//
	if (ThisNode->GetTree(0) != NNULL)
	{
		ThisClass = VARCLASS_ARRAY;
		ParCount = ThisNode->GetTree(0)->CountParam();
	}

	if ((ThisNode->GetType() == BAS_V_FUNCTION) ||
		(ThisNode->GetType() == BAS_S_MAINFUNCTION) ||
		(ThisNode->GetType() == BAS_S_FUNCTION))
	{
		ThisClass = VARCLASS_FUNC;
	}

	//
	// Add the variable to the list
	//
	VariableStruct NewVariable(
		TextName, Type, ThisClass,
		ThisOutput | PreDefined, ParCount);
	Append(NewVariable, GlobalFlag);
}


/*******************************************************************************

Member Function:

	Append()

Title:

	Add a variable to the table.

Description:

	This routine will add a variable to the end of the variable table.

Bugs:

	Assumes variable is not already in the table.

*******************************************************************************/

void VariableList::Append(const VariableStruct& Variable, int GlobalFlag)
{
	string keyname(Variable.BasicName);

	if ((Variable.Class == VARCLASS_ARRAY) ||
		(Variable.Class == VARCLASS_FUNC))
	{
		keyname += "(";
	}

	//
	// Decide which table to stick it on
	//
	if (GlobalFlag)
	{
		front()[keyname] = Variable;
	}
	else
	{
		if (back().find(keyname) != back().end())
		{
			cerr << "Putting in duplicate variable " << keyname << endl;
		}
		back()[keyname] = Variable;
	}
}


/*******************************************************************************

Member Function:

	VariableList::Fixup()

Title:

	Fix up variable names

Description:

	This function changes all of the variable names from
	formats that BASIC likes, into those that C likes.

Algorithm:

	1. Convert those variables that can be to simply lower case
	   versions of the upper case basic names.

	2. For variables with trailing characters "%", try leaving
	   the "%" off, and make sure there are no duplicates.

	3. Any variables left over, convert to lower case, changing
	   unallowed characters to underscores. Make sure that they
	   don't duplicate any other name.

	4. Additional to step 3, add a "Vn" to the variable to make
	   unique.

Bugs:

*******************************************************************************/

void VariableList::Fixup()
{
	for (reverse_iterator loop = rbegin();
		loop != rend(); loop++)
	{
		//
		// Do the various stages of fixup for global functions
		//
		Fixup1(&(*loop));
	}
}

/*******************************************************************************

Member Function:

	Fixup1()

Title:

	Fix up variable names

Description:

	Convert those variables that can be to simply lower case
	versions of the upper case basic names.

Bugs:

	I would like to move this into the ListOfVariables class,
	but it needs to be able to search through the entire list
	of all the levels, so it is stuck here.

*******************************************************************************/

void VariableList::Fixup1(ListOfVariables* Variable)
{
	//
	// Local Variables
	//
	string tempname;
	char number[6];

	//
	// Loop through all variables
	//
	for (ListOfVariables::iterator i = Variable->begin();
		i != Variable->end(); i++)
	{
		VariableStruct* VarList = &((*i).second);

		//
		// Don't re-convert, don't allow illegal basic characters.
		//
		if (VarList->CName.length() == 0)
		{
			if ((VarList->Type == VARTYPE_LABEL) &&
				(isdigit(VarList->BasicName[0])))
			{
				//
				// Lables are never reserved, so always
				// give them a specific value
				//
				tempname = "L_" + VarList->BasicName;
				for (int j = 2; j < tempname.length(); j++)
				{
					tempname[j] = tolower(tempname[j]);
				}
				VarList->CName = tempname;
			}
			else
			{
				tempname = VarList->BasicName;

				if (tempname[tempname.length()-1] == '(')
				{
					tempname = tempname.substr(0, tempname.length() - 1);
				}
				if ((tempname[tempname.length()-1] == '%') ||
					(tempname[tempname.length()-1] == '$'))
				{
					tempname = tempname.substr(0, tempname.length() - 1);
				}

				for (int j = 0; j < tempname.length(); j++)
				{
					tempname[j] = tolower(tempname[j]);
					if (tempname[j] == '.')
					{
						tempname[j] = '_';
					}
				}


				//
				// Don't assign if used by another variable
				//
				if (LookupReserved(tempname) == FALSE)
				{
					VarList->CName = tempname;
				}
				else
				{
					tempname += "_V";
					sprintf(number, "%d", vcount++);
					tempname += number;

					VarList->CName = tempname;
				}
			}
		}
	}
}

/*******************************************************************************

Member Function:

	VariableList::OutputDef()

Title:

	Output any variables that have not yet been dumped.

Description:

	This function will output any variables that have not been dumped
	by any previous routines.

Bugs:


*******************************************************************************/

void ListOfVariables::OutputDef(ostream& os, int Level)
{
	//
	// Loop through all variables
	//
	for (iterator i = begin(); i != end(); i++)
	{
		//
		// Output this variable
		//
		if ((*i).second.Output != TRUE)
		{ 
			os << string(Level, '\t') <<
				(*i).second.OutputDef() << ";" << endl;
		}
	}
}

/*******************************************************************************

Local Function:

	VariableList::InitFunctions()

Title:

	Initialize function names.

Description:

	This function initializes several built in basic functions
	that have compatible C functions (same parameters).

Bugs:


*******************************************************************************/

void VariableList::InitFunctions(void)
{
	InitOneFunction("ABS",		"fabs",		VARTYPE_DOUBLE);
	InitOneFunction("ABS%",		"abs",		VARTYPE_DOUBLE);
	InitOneFunction("ASC",		"Ascii",	VARTYPE_LONG);
	InitOneFunction("ASCII",	"Ascii",	VARTYPE_LONG);
	InitOneFunction("ATN",		"atan",		VARTYPE_DOUBLE);
	InitOneFunction("BUFSIZ",	"Bufsiz",	VARTYPE_LONG);
	InitOneFunction("CCPOS",	"Ccpos",	VARTYPE_LONG);
	InitOneFunction("CHR$",		"Char",		VARTYPE_DYNSTR);
	InitOneFunction("COMP%",	"Comp",		VARTYPE_LONG);
	InitOneFunction("COS",		"cos",		VARTYPE_DOUBLE);
	InitOneFunction("CTRLC",	"ctrlc()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("CVT$%",	"cvtai",	VARTYPE_LONG);
	InitOneFunction("CVT%$",	"cvtia",	VARTYPE_DYNSTR);
	InitOneFunction("CVT$F",	"cvtaf",	VARTYPE_DOUBLE);
	InitOneFunction("CVTF$",	"cvtfa",	VARTYPE_DYNSTR);
	InitOneFunction("CVT$$",	"edit",		VARTYPE_DYNSTR);
	InitOneFunction("DATE$",	"Qdate",	VARTYPE_DYNSTR);
	InitOneFunction("DET",		"det()",	VARTYPE_DOUBLE, VARCLASS_NONE);
	InitOneFunction("DIF$",		"Dif",		VARTYPE_DYNSTR);
	InitOneFunction("ECHO",		"Echo",		VARTYPE_LONG);
	InitOneFunction("EDIT$",	"edit",		VARTYPE_DYNSTR);
	InitOneFunction("ERR",		"err()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("ERL",		"erl()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("ERT$",		"ert",		VARTYPE_DYNSTR);
	InitOneFunction("EXP",		"exp",		VARTYPE_DOUBLE);
	InitOneFunction("FIX",		"fix",		VARTYPE_DOUBLE);
	InitOneFunction("FORMAT$",	"Format",	VARTYPE_DYNSTR);
	InitOneFunction("FSP$",		"Fsp",		VARTYPE_DYNSTR);
	InitOneFunction("FSS$",		"Fss",		VARTYPE_DYNSTR);
	InitOneFunction("GETRFA",	"Getrfa",	VARTYPE_RFA);
	InitOneFunction("INKEY$",	"Inkey",	VARTYPE_DYNSTR);
	InitOneFunction("INSTR",	"instr",	VARTYPE_LONG);
	InitOneFunction("INT",		"floor",	VARTYPE_DOUBLE);
	InitOneFunction("LEFT",		"left",		VARTYPE_DYNSTR);
	InitOneFunction("LEFT$",	"left",		VARTYPE_DYNSTR);
	InitOneFunction("LEN",		"strlen",	VARTYPE_LONG);
	InitOneFunction("LOC",		"&",		VARTYPE_LONG);
	InitOneFunction("LOG",		"log",		VARTYPE_DOUBLE);
	InitOneFunction("LOG10",	"log10",	VARTYPE_DOUBLE);
	InitOneFunction("MAG",		"Mag",		VARTYPE_DOUBLE);
	InitOneFunction("MAGTAPE",	"Magtape",	VARTYPE_LONG);
	InitOneFunction("MAR",		"Mar",		VARTYPE_LONG);
	InitOneFunction("MID",		"mid",		VARTYPE_DYNSTR);
	InitOneFunction("MID$",		"mid",		VARTYPE_DYNSTR);
	InitOneFunction("NOECHO",	"NoEcho",	VARTYPE_LONG);
	InitOneFunction("NUM",		"num()",	VARTYPE_DOUBLE, VARCLASS_NONE);
	InitOneFunction("NUM2",		"num2()",	VARTYPE_DOUBLE, VARCLASS_NONE);
	InitOneFunction("NUM$",		"Qnum",		VARTYPE_DYNSTR);
	InitOneFunction("NUM1$",	"Qnum1",	VARTYPE_DYNSTR);
	InitOneFunction("ONECHR",	"OneChr",	VARTYPE_LONG);
	InitOneFunction("PLACE$",	"Place",	VARTYPE_DYNSTR);
	InitOneFunction("POS",		"Ccpos",	VARTYPE_LONG);
	InitOneFunction("PROD$",	"Prod",		VARTYPE_DYNSTR);
	InitOneFunction("QUO$",		"Quo",		VARTYPE_DYNSTR);
	InitOneFunction("RAD$",		"Qrad",		VARTYPE_DYNSTR);
	InitOneFunction("RCTLC",	"rctlc()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("RCTLO",	"rctlo()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("RECOUNT",	"recount()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("RIGHT",	"right",	VARTYPE_DYNSTR);
	InitOneFunction("RIGHT$",	"right",	VARTYPE_DYNSTR);
	InitOneFunction("RND",		"floatrand",	VARTYPE_DOUBLE);
	InitOneFunction("RND",		"floatrand()",	VARTYPE_DOUBLE, VARCLASS_NONE);
	InitOneFunction("SEG$",		"Qseg",		VARTYPE_DYNSTR);
	InitOneFunction("SGN",		"sgn",		VARTYPE_LONG);
	InitOneFunction("SIN",		"sin",		VARTYPE_DOUBLE);
	InitOneFunction("SPACE$",	"Qspace",	VARTYPE_DYNSTR);
	InitOneFunction("SQRT",		"sqrt",		VARTYPE_DOUBLE);
	InitOneFunction("SQR",		"sqrt",		VARTYPE_DOUBLE);
	InitOneFunction("STATUS",	"status()",	VARTYPE_LONG, VARCLASS_NONE);
	InitOneFunction("STR$",		"Qnum",		VARTYPE_DYNSTR);
	InitOneFunction("STRING$",	"Qstring",	VARTYPE_DYNSTR);
	InitOneFunction("SUM$",		"Sum",		VARTYPE_DYNSTR);
	InitOneFunction("SWAP%",	"Swap",		VARTYPE_LONG);
	InitOneFunction("SYS",		"sys",		VARTYPE_DYNSTR);
	InitOneFunction("TAB",		"Tab",		VARTYPE_DOUBLE);
	InitOneFunction("TAN",		"tan",		VARTYPE_DOUBLE);
	InitOneFunction("TIME",		"Time",		VARTYPE_DOUBLE);
	InitOneFunction("TIME$",	"Qtime",	VARTYPE_DYNSTR);
	InitOneFunction("TRM$",		"trm",		VARTYPE_DYNSTR);
	InitOneFunction("VAL",		"atof",		VARTYPE_DOUBLE);
	InitOneFunction("VAL%",		"atoi",		VARTYPE_LONG);
	InitOneFunction("XLATE",	"Xlate",	VARTYPE_DYNSTR);
}

/*******************************************************************************

Local Function:

	VariableList::InitOneFunction()

Title:

	Initialize function names.

Description:

	This function fills in remaining blanks for a function.

Bugs:
	This function is defined to be used be InitFunctions only.


*******************************************************************************/

void VariableList::InitOneFunction(string BasName, char *CeeName,
	int Type, int Class)
{
	VariableStruct ThisVar(BasName, Type, Class, 1);
	ThisVar.SetCName(CeeName);
	Append(ThisVar, TRUE);
}

